package com.famoco.barcodesampleapp.repository.scanner

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Bundle
import android.util.Log
import android.view.KeyEvent
import kotlinx.coroutines.*

class ScannerManager320(context: Context) : ScannerManager(context) {

    companion object {
        private val TAG = ScannerManager320::class.simpleName
        const val KEY_LASER = 280

        private const val SCAN_COMPLETE_ACTION = "com.zebra.sdl.action.SCAN_COMPLETE"
        private const val BARCODE_CONTENT_EXTRA = "com.zebra.sdl.extra.CONTENT"
        private const val ENABLE_SCANNER_ACTION = "com.zebra.sdl.action.ENABLE"
        private const val DISABLE_SCANNER_ACTION = "com.zebra.sdl.action.DISABLE"
        private const val START_SCAN_ACTION = "com.zebra.sdl.action.START"
        private const val START_SCAN_PARAMETERS_EXTRA = "com.zebra.sdl.extra.PARAMETERS"
        private const val STOP_SCAN_ACTION = "com.zebra.sdl.action.STOP"
        private const val ZEBRA_PACKAGE = "com.zebra.sdl"
    }

    private val scanReceiver = object: BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            val scanResult = intent?.extras?.getString(BARCODE_CONTENT_EXTRA)
            Log.i(TAG, "barcode scan received: $scanResult")
            if (_enableFillEditText.value == true) {
                coroutineScope.launch {
                    if (scanResult == null) {
                        return@launch
                    }
                    sendKeyEvents(scanResult)
                }
            }
            _lastScanBarcodeResult.postValue(scanResult)
            if (_nonStopScan.value == false) {
                _scanning.postValue(false)
            }
        }
    }

    init {
        val intentFilter = IntentFilter(SCAN_COMPLETE_ACTION)
        context.registerReceiver(scanReceiver, intentFilter)
    }

    override suspend fun enableScanner() {
        Log.i(TAG, "enable scanner")
        disableScanner() // to avoid any conflict
        val intent = Intent(ENABLE_SCANNER_ACTION)
        intent.`package` = ZEBRA_PACKAGE
        intent.flags = Intent.FLAG_FROM_BACKGROUND
        context.startService(intent)
    }

    override suspend fun disableScanner() {
        Log.i(TAG, "disable scanner")
        val intent = Intent(DISABLE_SCANNER_ACTION)
        intent.`package` = ZEBRA_PACKAGE
        intent.flags = Intent.FLAG_FROM_BACKGROUND
        context.startService(intent)
    }

    override suspend fun startScan() {
        Log.i(TAG, "start scan")
        _scanning.postValue(true)
        // set all parameters
        val parameters = Bundle()
        parameters.putInt("12", 1) // UPC-E1
        parameters.putInt("83", 1) // BOOKLAND_EAN
        parameters.putInt("85", 1) // UCC_COUPON_EXTENDED_CODE
        parameters.putInt("617", 1) // ISSN_EAN
        parameters.putInt("9", 1) // CODE_93
        parameters.putInt("10", 1) // CODE_11
        parameters.putInt("5", 1) // DISCRETE_2_TO_5
        parameters.putInt("35", 1) // DISCRETE_2_TO_5_INTERVAL
        parameters.putInt("7", 1) // CODABAR
        parameters.putInt("11", 1) // MSI
        parameters.putInt("341", 1) // COMPOSITE_CC_C
        parameters.putInt("342", 1) // COMPOSITE_CC_A_B
        parameters.putInt("371", 1) // COMPOSITE_TLS_39
        parameters.putInt("227", 1) // MICROPDF417
        parameters.putInt("537", 1) // DECODE_MIRROR_IMAGE_DATA_MATRIX_ONLY
        parameters.putInt("589", 1) // AZTEC_INVERSE
        parameters.putInt("1167", 1) // HANXIN
        parameters.putInt("1168", 1) // HANXIN_INVERSE

        val intent = Intent(START_SCAN_ACTION)
        intent.`package` = ZEBRA_PACKAGE
        intent.flags = Intent.FLAG_FROM_BACKGROUND
        intent.putExtra(START_SCAN_PARAMETERS_EXTRA, parameters)
        context.startService(intent)
    }

    override suspend fun stopScan() {
        Log.i(TAG, "stop scan")
        _scanning.postValue(false)
        val intent = Intent(STOP_SCAN_ACTION)
        intent.`package` = ZEBRA_PACKAGE
        intent.flags = Intent.FLAG_FROM_BACKGROUND
        context.startService(intent)
    }

    override suspend fun clearBarcodeData() {
        Log.i(TAG, "clear barcode data")
        stopScan()
        _lastScanBarcodeResult.postValue("")
    }

    override suspend fun enableFillEditText(enable: Boolean) {
        Log.i(TAG, "enable fill edit text")
        stopScan()
        _enableFillEditText.postValue(enable)
    }

    override suspend fun setContinuousScanPhysicalButton(enable: Boolean) {
        Log.i(TAG, "set continuous scan physical button")
        stopScan()
        _continuousScanPhysicalButton.postValue(enable)
    }

    override suspend fun setNonStopScan(enable: Boolean) {
        Log.i(TAG, "set continuous scan physical button")
        Log.w(TAG, "setNonStopScan: Not implemented")
        stopScan()
    }

    override suspend fun handlePhysicalButtonPress(keyCode: Int, event: KeyEvent?) {
        // nothing to do
    }

    override suspend fun handlePhysicalButtonRelease(keyCode: Int, event: KeyEvent?) {
        if (keyCode != KEY_LASER) {
            return
        }

        if (continuousScanPhysicalButton.value == true) {
            if (scanning.value == false)
                startScan()
            else {
                stopScan()
            }
        }
    }
}