package com.famoco.barcodesampleapp.repository.scanner

import android.content.Context
import android.graphics.SurfaceTexture
import android.graphics.SurfaceTexture.OnFrameAvailableListener
import android.hardware.camera2.CameraAccessException
import android.hardware.camera2.CameraManager
import android.provider.Settings
import android.provider.Settings.SettingNotFoundException
import android.util.Log
import android.view.KeyEvent
import androidx.core.content.ContextCompat.getSystemService
import com.famoco.barcode.BarcodeManager
import com.zebra.adc.decoder.BarCodeReader
import com.zebra.adc.decoder.BarCodeReader.DecodeCallback
import kotlinx.coroutines.launch

class ScannerManager325(context: Context) : ScannerManager(context), DecodeCallback,
    BarCodeReader.ErrorCallback, OnFrameAvailableListener {

    companion object {
        private val TAG = ScannerManager325::class.simpleName

        private const val BARCODE_SCANNER_SERVICE_ENABLED = "barcode_service_enabled"
        var BARCODE_READER_PARAMS = arrayOf(
            intArrayOf(687, 4),
            intArrayOf(764, 0),
            intArrayOf(765, 0),
            intArrayOf(8610, 1),
            intArrayOf(8611, 0)
        )
    }

    private var libraryLoaded = false
    private val barcodeManager: BarcodeManager
    private var barcodeReader: BarCodeReader? = null
    private var surfaceTexture: SurfaceTexture? = null

    init {
        try {
            System.loadLibrary("IAL.zebra")
            System.loadLibrary("SDL.zebra")
            System.loadLibrary("barcodereader90.zebra")
            libraryLoaded = true
        } catch (e: Exception) {
            libraryLoaded = false
            Log.e(TAG, e.message, e)
        }

        barcodeManager = BarcodeManager.getInstance(context)
    }

    private fun isBarcodeServiceEnabled(): Boolean {
        try {
            return Settings.Global.getInt(
                context.contentResolver,
                BARCODE_SCANNER_SERVICE_ENABLED
            ) == 1
        } catch (e: SettingNotFoundException) {
            Log.e(TAG, e.message, e)
        }
        return false
    }

    @Throws(CameraAccessException::class)
    private fun getScannerCameraId(): Int {
        return getNumberOfCameras() - 1
    }

    @Throws(CameraAccessException::class)
    private fun getNumberOfCameras(): Int {
        val manager = getSystemService(context, CameraManager::class.java) as CameraManager
        return manager.cameraIdList.size
    }

    private fun setBarcodeParameters(params: Array<IntArray>) {
        for (param in params) {
            setBarcodeParameter(param[0], param[1])
        }
    }

    private fun setBarcodeParameter(paramId: Int, paramValue: Int) {
        val res = barcodeReader?.setParameter(paramId, paramValue)
        if (res != BarCodeReader.BCR_SUCCESS) {
            Log.e(TAG, "Failed to set barcode parameter $paramId with value = $paramValue")
            return
        }
    }

    override suspend fun enableScanner() {
        Log.i(TAG, "enable scanner")
        if (isBarcodeServiceEnabled() && barcodeManager.isActive) {
            barcodeManager.pause() // to avoid any conflict
        }

        try {
            val readerId: Int = getScannerCameraId()
            Log.i(TAG, "Open reader with Camera Id = $readerId")
            barcodeReader = BarCodeReader.open(readerId, context)
            barcodeReader?.setDecodeCallback(this)
            barcodeReader?.setErrorCallback(this)
            setBarcodeParameters(BARCODE_READER_PARAMS)

            // The new SDK requires set preview texture
            surfaceTexture = SurfaceTexture(5)
            surfaceTexture?.setOnFrameAvailableListener(this)
            barcodeReader?.setPreviewTexture(surfaceTexture)
        } catch (e: java.lang.Exception) {
            Log.e(TAG, e.message, e)
        }
    }

    override suspend fun disableScanner() {
        Log.i(TAG, "disable scanner")
        barcodeReader?.release()
        barcodeReader = null

        if (isBarcodeServiceEnabled() && !barcodeManager.isActive) {
            // make life easier for other apps
            barcodeManager.resume()
        }
    }

    override suspend fun startScan() {
        Log.i(TAG, "start scan")
        _scanning.postValue(true)
        val status = barcodeReader?.getNumProperty(BarCodeReader.PropertyNum.ENGINE_STATUS)
        status?.let { Log.d(TAG, "Engine Status 0x" + Integer.toHexString(it)) }
        val res = if (_nonStopScan.value == true) {
            barcodeReader?.startHandsFreeDecode(BarCodeReader.ParamVal.HANDSFREE.toInt())
        } else {
            barcodeReader?.startDecode()
        }
        Log.i(TAG, "Decoding finished: result = $res")
    }

    override suspend fun stopScan() {
        Log.i(TAG, "stop scan")
        _scanning.postValue(false)
        barcodeReader?.stopDecode()
    }

    override suspend fun clearBarcodeData() {
        Log.i(TAG, "clear barcode data")
        stopScan()
        _lastScanBarcodeResult.postValue("")
    }

    override suspend fun enableFillEditText(enable: Boolean) {
        Log.i(TAG, "enable fill edit text")
        stopScan()
        _enableFillEditText.postValue(enable)
    }

    override suspend fun setContinuousScanPhysicalButton(enable: Boolean) {
        Log.i(TAG, "set continuous scan physical button")
        stopScan()
        _continuousScanPhysicalButton.postValue(enable)
    }

    override suspend fun setNonStopScan(enable: Boolean) {
        Log.w(TAG, "set non-stop scan")
        stopScan()
        _nonStopScan.postValue(enable)
    }

    override suspend fun handlePhysicalButtonPress(keyCode: Int, event: KeyEvent?) {
        if ((keyCode == KeyEvent.KEYCODE_SHIFT_LEFT || keyCode == KeyEvent.KEYCODE_F1 || keyCode == KeyEvent.KEYCODE_F2)
            && event!!.repeatCount == 0
        ) {
            if (continuousScanPhysicalButton.value != true) {
                startScan()
            }
        }
    }

    override suspend fun handlePhysicalButtonRelease(keyCode: Int, event: KeyEvent?) {
        if (keyCode == KeyEvent.KEYCODE_SHIFT_LEFT || keyCode == KeyEvent.KEYCODE_F1 || keyCode == KeyEvent.KEYCODE_F2) {
            if (continuousScanPhysicalButton.value == true) {
                if (scanning.value == false)
                    startScan()
                else {
                    stopScan()
                }
            } else {
                stopScan()
            }
        }
    }

    override fun onDecodeComplete(
        symbology: Int,
        length: Int,
        data: ByteArray?,
        reader: BarCodeReader?
    ) {
        Log.i(TAG, "onDecodeComplete")
        if (length <= 0 || data == null || data.isEmpty()) {
            return
        }
        try {
            val decodedText: String = String(data, 0, length)
            handleScanResult(decodedText)
        } catch (e: java.lang.Exception) {
            Log.e(TAG, e.message, e)
        } finally {
            if (_nonStopScan.value == false) {
                barcodeReader?.stopDecode()
            }
        }
    }

    override fun onEvent(p0: Int, p1: Int, p2: ByteArray?, p3: BarCodeReader?) {
        // nothing to do
    }

    private fun handleScanResult(decodedText: String) {
        if (_nonStopScan.value == false) {
            _scanning.postValue(false)
        }
        if (_enableFillEditText.value == true) {
            coroutineScope.launch {
                sendKeyEvents(decodedText)
            }
        }

        _lastScanBarcodeResult.postValue(decodedText)
    }

    override fun onError(p0: Int, p1: BarCodeReader?) {
        // nothing to do
    }

    override fun onFrameAvailable(surfaceTexture: SurfaceTexture?) {
        // nothing to do
    }
}